package com.zhilian.common.redis.service;

import io.lettuce.core.output.DoubleListOutput;
import org.apache.poi.ss.formula.functions.T;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.*;
import org.springframework.stereotype.Component;

import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * spring redis 工具类
 *
 * @author zhilian
 **/
@SuppressWarnings(value = {"unchecked", "rawtypes"})
@Component
public class RedisService {
    @Autowired
    public RedisTemplate redisTemplate;

    /**
     * 缓存基本的对象，Integer、String、实体类等
     *
     * @param key   缓存的键值
     * @param value 缓存的值
     */
    public <T> void setCacheObject(final String key, final T value) {
        redisTemplate.opsForValue().set(key, value);
    }

    /**
     * 缓存基本的对象，Integer、String、实体类等
     *
     * @param key      缓存的键值
     * @param value    缓存的值
     * @param timeout  时间
     * @param timeUnit 时间颗粒度
     */
    public <T> void setCacheObject(final String key, final T value, final Long timeout, final TimeUnit timeUnit) {
        redisTemplate.opsForValue().set(key, value, timeout, timeUnit);
    }

    /**
     * 设置有效时间
     *
     * @param key     Redis键
     * @param timeout 超时时间
     * @return true=设置成功；false=设置失败
     */
    public boolean expire(final String key, final long timeout) {
        return expire(key, timeout, TimeUnit.SECONDS);
    }

    /**
     * 设置有效时间
     *
     * @param key     Redis键
     * @param timeout 超时时间
     * @param unit    时间单位
     * @return true=设置成功；false=设置失败
     */
    public boolean expire(final String key, final long timeout, final TimeUnit unit) {
        return redisTemplate.expire(key, timeout, unit);
    }

    /**
     * 获取有效时间
     *
     * @param key Redis键
     * @return 有效时间
     */
    public long getExpire(final String key) {
        return redisTemplate.getExpire(key);
    }

    /**
     * 判断 key是否存在
     *
     * @param key 键
     * @return true 存在 false不存在
     */
    public Boolean hasKey(String key) {
        return redisTemplate.hasKey(key);
    }

    /**
     * 获得缓存的基本对象。
     *
     * @param key 缓存键值
     * @return 缓存键值对应的数据
     */
    public <T> T getCacheObject(final String key) {
        ValueOperations<String, T> operation = redisTemplate.opsForValue();
        return operation.get(key);
    }

    /**
     * 删除单个对象
     *
     * @param key
     */
    public boolean deleteObject(final String key) {
        return redisTemplate.delete(key);
    }

    /**
     * 删除集合对象
     *
     * @param collection 多个对象
     * @return
     */
    public boolean deleteObject(final Collection collection) {
        return redisTemplate.delete(collection) > 0;
    }

    /**
     * 缓存List数据
     *
     * @param key      缓存的键值
     * @param dataList 待缓存的List数据
     * @return 缓存的对象
     */
    public <T> long setCacheList(final String key, final List<T> dataList) {
        Long count = redisTemplate.opsForList().rightPushAll(key, dataList);
        return count == null ? 0 : count;
    }

    /**
     * 获得缓存的list对象
     *
     * @param key 缓存的键值
     * @return 缓存键值对应的数据
     */
    public <T> List<T> getCacheList(final String key) {
        return redisTemplate.opsForList().range(key, 0, -1);
    }

    /**
     * 缓存Set
     *
     * @param key     缓存键值
     * @param dataSet 缓存的数据
     * @return 缓存数据的对象
     */
    public <T> BoundSetOperations<String, T> setCacheSet(final String key, final Set<T> dataSet) {
        BoundSetOperations<String, T> setOperation = redisTemplate.boundSetOps(key);
        Iterator<T> it = dataSet.iterator();
        while (it.hasNext()) {
            setOperation.add(it.next());
        }
        return setOperation;
    }

    /**
     * 获得缓存的set
     *
     * @param key
     * @return
     */
    public <T> Set<T> getCacheSet(final String key) {
        return redisTemplate.opsForSet().members(key);
    }

    /**
     * 缓存Map
     *
     * @param key
     * @param dataMap
     */
    public <T> void setCacheMap(final String key, final Map<String, T> dataMap) {
        if (dataMap != null) {
            redisTemplate.opsForHash().putAll(key, dataMap);
        }
    }

    /**
     * 获得缓存的Map
     *
     * @param key
     * @return
     */
    public <T> Map<String, T> getCacheMap(final String key) {
        return redisTemplate.opsForHash().entries(key);
    }

    /**
     * 往Hash中存入数据
     *
     * @param key   Redis键
     * @param hKey  Hash键
     * @param value 值
     */
    public <T> void setCacheMapValue(final String key, final String hKey, final T value) {
        redisTemplate.opsForHash().put(key, hKey, value);
    }

    /**
     * 获取Hash中的数据
     *
     * @param key  Redis键
     * @param hKey Hash键
     * @return Hash中的对象
     */
    public <T> T getCacheMapValue(final String key, final String hKey) {
        HashOperations<String, String, T> opsForHash = redisTemplate.opsForHash();
        return opsForHash.get(key, hKey);
    }

    /**
     * 获取多个Hash中的数据
     *
     * @param key   Redis键
     * @param hKeys Hash键集合
     * @return Hash对象集合
     */
    public <T> List<T> getMultiCacheMapValue(final String key, final Collection<Object> hKeys) {
        return redisTemplate.opsForHash().multiGet(key, hKeys);
    }

    /**
     * 删除Hash中的某条数据
     *
     * @param key  Redis键
     * @param hKey Hash键
     * @return 是否成功
     */
    public boolean deleteCacheMapValue(final String key, final String hKey) {
        return redisTemplate.opsForHash().delete(key, hKey) > 0;
    }


    /**
     * 向Zset中存入单条数据
     *
     * @param zkey  ZSet键
     * @param value 值
     * @param score 分数
     */
    public <T> void setCacheZsetValue(final String zkey, final T value, final Double score) {
        redisTemplate.opsForZSet().add(zkey, value, score);
    }

    /**
     * 向Zset中存入多条数据
     *
     * @param zkey zSet键
     * @param map  => 键值对 键T  值Double分数
     */
    public <T> void setCacheZsetBatchValue(final String zkey, final Map<T, Double> map) {
        Set<DefaultTypedTuple> collect = map.entrySet().stream()
                .map(item -> {
                    return new DefaultTypedTuple(item.getKey(), item.getValue());
                })
                .collect(Collectors.toSet());

        redisTemplate.opsForZSet().add(zkey, collect);
    }

    /**
     * 获取Zset集合大小
     *
     * @param zkey
     * @return size
     */
    public Long getCacheZsetSize(String zkey) {
        return redisTemplate.opsForZSet().size(zkey);
    }

    /**
     * 查询成员分数
     *
     * @param zkey
     * @param value
     * @param <T>
     * @return score
     */
    public <T> Double getCacheZsetScore(final String zkey, final T value) {
        return redisTemplate.opsForZSet().score(zkey, value);
    }

    /**
     * 范围查询分数
     *
     * @param zkey
     * @param start
     * @param end
     * @return
     */
    public <T> Set<Double> getCacheZsetScoreBatch(final String zkey, final Long start, final Long end) {
        return redisTemplate.opsForZSet().range(zkey, start, end);
    }

    /**
     * 范围查询集合携带分数
     * @param zkey
     * @param start
     * @param end
     * @return
     */
    public <T> Set<HashMap<Object, Double>> getCacheZsetScoreWithDataBatch(final String zkey, final Long start, final Long end) {
        Set<ZSetOperations.TypedTuple<Object>> rangeSet = redisTemplate.opsForZSet().range(zkey, start, end);
        Set<HashMap<Object, Double>> collect = rangeSet.stream().
                map(item -> {
                    HashMap<Object, Double> objectDoubleHashMap = new HashMap<Object, Double>();
                    objectDoubleHashMap.put(item.getValue(), item.getScore());
                    return objectDoubleHashMap;
                })
                .collect(Collectors.toSet());
        return collect;
    }

    /**
     * 增加分数
     * @param zkey
     * @param value
     * @param delta
     * @param <T>
     */
    public <T> Double incrementScore(final String zkey, final T value, final Double delta){
        return redisTemplate.opsForZSet().incrementScore(zkey,value,delta);
    }

    /**
     * 删除元素
     * @param zkey
     * @param values
     * @return
     * @param <T>
     */
    public <T> Long removeCacheZsetBatch(final String zkey,final T... values){
        return redisTemplate.opsForZSet().remove(zkey,values);
    }

    /**
     * 排序并限制结果集大小,按降序返回有序集合的部分元素，支持指定索引范围。
     * @param zkey
     * @param start
     * @param end
     * @return
     */
    public Set<Double> reverseRange(final String zkey,final Long start,final Long end){
        return redisTemplate.opsForZSet().reverseRange(zkey,start,end);
    }

    /**
     * 按分数范围查询
     * @param zkey
     * @param min
     * @param max
     * @return
     */
    public Set<Double> rangeByScore(final String zkey,final Double min,final Double max){
        return redisTemplate.opsForZSet().rangeByScore(zkey,min,max);
    }

    /**
     * 获取最小值
     * @param zkey
     * @return
     */
    public Map<Object, Double> getCacheZsetMin(final String zkey){
        ZSetOperations.TypedTuple typedTuple = redisTemplate.opsForZSet().popMin(zkey);
        return new HashMap<Object,Double>(){{put(typedTuple.getValue(),typedTuple.getScore());}};
    }


    /**
     * 获得缓存的基本对象列表
     *
     * @param pattern 字符串前缀
     * @return 对象列表
     */
    public Collection<String> keys(final String pattern) {
        return redisTemplate.keys(pattern);
    }
}
