package com.zhilian.online.config;/**
 * @version:
 * @Author: LiYuan
 * @description:
 * @date: 2024/3/30  19:33
 */

import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.core.Binding;
import org.springframework.amqp.core.BindingBuilder;
import org.springframework.amqp.core.DirectExchange;
import org.springframework.amqp.core.Queue;
import org.springframework.amqp.rabbit.connection.ConnectionFactory;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.amqp.support.converter.Jackson2JsonMessageConverter;
import org.springframework.amqp.support.converter.MessageConverter;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

import java.util.HashMap;
import java.util.Map;
import java.util.Objects;

/**
 * @BelongsProject: smart-cloud-server
 * @BelongsPackage: com.zhilian.online.config
 * @Author: LiYuan
 * @CreateTime: 2024-03-30  19:33
 * @Description: 负载中心RabbitMQ配置
 * @Version: 1.0
 */
@Configuration
@Slf4j
public class RabbitConfig {

    /**
     * 延迟队列名称
     */
    public static final String DELAY_QUEUE_NAME = "delay_queue";

    /**
     * 延迟交换机名称
     */
    public static final String DELAY_EXCHANGE_NAME = "delay_exchange";

    /**
     * 延迟路由键
     */
    public static final String DELAY_ROUTING_KEY = "delay_routing";

    /**
     * 死信队列名称
     */
    public static final String DEAD_QUEUE_NAME = "dead_queue";

    /**
     * 死信交换机名称
     */
    public static final String DEAD_EXCHANGE_NAME = "dead_exchange";

    /**
     * 死信路由键
     */
    public static final String DEAD_ROUTING_KEY = "dead_routing";

    /**
     * 注入死信队列实例
     *
     * @return Queue
     */
    @Bean
    Queue deadQueue() {
        log.info("死信队列创建成功");
        return new Queue(DEAD_QUEUE_NAME, true, false, false);
    }

    /**
     * 注入死信交换机实例
     *
     * @return DirectExchange
     */
    @Bean
    DirectExchange deadExchange() {
        log.info("死信交换机创建成功");
        return new DirectExchange(DEAD_EXCHANGE_NAME, true, false);
    }

    /**
     * 将死信队列与死信交换机绑定
     *
     * @return Binding
     */
    @Bean
    Binding deadBinding() {
        log.info("死信通道建立成功");
        return BindingBuilder.bind(deadQueue())
                .to(deadExchange())
                .with(DEAD_ROUTING_KEY);
    }


    /**
     * 注入延迟队列实例
     *
     * @return Queue
     */
    @Bean
    Queue delayQueue() {
        Map<String, Object> map = new HashMap<>();
        //设置消息过期时间为30秒
        map.put("x-message-ttl", 1000 * 30);
        //设置消息过期后存储的死信交换机
        map.put("x-dead-letter-exchange", DEAD_EXCHANGE_NAME);
        //设置死信路由键
        map.put("x-dead-letter-routing-key", DEAD_ROUTING_KEY);
        log.info("延迟队列创建成功");
        return new Queue(DELAY_QUEUE_NAME, true, false, false, map);
    }

    /**
     * 注入延迟交换机实例
     *
     * @return DirectExchange
     */
    @Bean
    DirectExchange delayExchange() {
        log.info("延迟交换机创建成功");
        return new DirectExchange(DELAY_EXCHANGE_NAME, true, false);
    }

    /**
     * 将延迟队列与延迟交换机绑定
     *
     * @return
     */
    @Bean
    Binding delayBinding() {
        log.info("延迟通道创建成功");
        return BindingBuilder.bind(deadQueue())
                .to(delayExchange())
                .with(DELAY_ROUTING_KEY);
    }

    /**
     * 配置RabbitMq序列化器
     * @param connectionFactory
     * @return
     */
//    @Bean
//    public RabbitTemplate rabbitTemplate(ConnectionFactory connectionFactory){
//        RabbitTemplate rabbitTemplate = new RabbitTemplate(connectionFactory);
//        rabbitTemplate.setMessageConverter(new Jackson2JsonMessageConverter());
//        return rabbitTemplate;
//    }



//    @Bean
//    public MessageConverter jsonMessageConverter(){
//        return new Jackson2JsonMessageConverter();
//    }


}
